#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>

#include <rpi_gpio.h>
#include <rpi_gpiolib.h>

#define LED_BASE	7	// LEDのベースGPIO番号

#define DIGIT_BASE	14	// 桁制御のベースGPIO番号
#define DIGITS		4	// 桁数

#define BLANK	10

// 管理データ
struct seg_data {
	int exit_flag;
	int *display_value;
	int hz;
} data;

// GPIO初期化
void led_init(void)
{
	// GPIOファンクションセット
	rpi_gpio_function_set( LED_BASE+0, RPI_GPF_OUTPUT );
	rpi_gpio_function_set( LED_BASE+1, RPI_GPF_OUTPUT );
	rpi_gpio_function_set( LED_BASE+2, RPI_GPF_OUTPUT );
	rpi_gpio_function_set( LED_BASE+3, RPI_GPF_OUTPUT );
	
	rpi_gpio_function_set( DIGIT_BASE  , RPI_GPF_OUTPUT);
	rpi_gpio_function_set( DIGIT_BASE+1, RPI_GPF_OUTPUT);
}

void led_put( unsigned int v )
{
	rpi_gpio_clear32( RPI_GPIO_P1MASK, 0x0F << LED_BASE);
	rpi_gpio_set32( RPI_GPIO_P1MASK, (v & 0x0F) << LED_BASE);
}


// 4桁表示スレッド

void *exec_4digits(void *param)
{
	static int dig;
	struct seg_data *d = (struct seg_data *)param;
	int usec = (1000000/DIGITS) / d->hz;	// 1桁あたりの待ち時間

	// GPIOポート初期化
	led_init();

	while(! d->exit_flag ) {
		if(++dig >= DIGITS) dig = 0;
		
		// Blankを入れる
		led_put(BLANK);
		// 点灯桁をシフトする
		rpi_gpio_clear32( RPI_GPIO_P1MASK, 0x03 << DIGIT_BASE);
		rpi_gpio_set32( RPI_GPIO_P1MASK, (dig ) << DIGIT_BASE);
		// 数字を出力
		led_put(d->display_value[dig]);
		usleep(usec);
	}
	
	return NULL;
}


void signal_handler(int sig)
{
	data.exit_flag = sig;
}

int main(void)
{
	int counter = 0;
	int display_value[4] = {0,0,0,0};
	pthread_t thread;
	
	// GPIO初期化
	if( ! rpi_gpio_init() )
		return -1;
	
	signal( SIGINT  , signal_handler );
	signal( SIGTERM , signal_handler );
	signal( SIGQUIT , signal_handler );
	signal( SIGKILL , signal_handler );
	
	data.exit_flag = 0;
	data.display_value = display_value;
	data.hz = 65;	// 1秒間に65回表示
	
	pthread_create( &thread, NULL, exec_4digits, &data );
	
	while( data.exit_flag == 0 ) {
		int c;
		
		if( counter > 9999 ) counter = 0;
		
		data.display_value[3] =  counter / 1000;
		if( data.display_value[3] == 0 ) data.display_value[3] = BLANK;
		c = counter % 1000;
		
		data.display_value[2] =  c / 100;
		if( (data.display_value[2] == 0 ) && (counter < 100) ) data.display_value[2] = BLANK;
		c = c % 100;
		
		data.display_value[1] =  c / 10;
		if( (data.display_value[1] == 0) && (counter < 10) ) data.display_value[1] = BLANK;

		data.display_value[0] =  c % 10;
		
		counter++;
		usleep(100*1000);
	}
	// スレッド終了待ち
	pthread_join(thread, NULL);
	rpi_gpio_deinit();
	
	return 1;
}
