#include "mcp320x.h"

MCP320X::MCP320X( int bus, int cs )
{
	try {
		this->spi = new SPI( bus, cs, SPI_CPOL | SPI_CPHA );
	}
	catch( SPIException e ) {
		throw e;
	}
	this->ref = 3.30;	// default = 3.3V
}

void MCP320X::setReference( double r )
{
	this->ref = r;
}

double MCP320X::getValue( int channel ) 
{
	try {
		return this->getValue( channel, MCP320X_SINGLE );
	}
	catch( SPIException e ) {
		throw e;
	}
}

double MCP320X::getValue(int channel, int s_d )
{
	uint8_t c;
	uint8_t *rxbuf;
	int d = 0;
	
	this->packet[0] = this->packet[1] = this->packet[2] = 0;
	this->packet[0] |= HIGH << 2;	// スタートビット
	
	// シングルエンデット，ディファレンシャル
	c = s_d & 0x01;
	this->packet[0] |= c << 1;
	
	// アナログチャンネル
	c = channel & 0x07;
	this->packet[0] |= (c & 0x04) >> 2;
	this->packet[1] |= (c & 0x03) << 6;
	
	try {
		rxbuf = this->spi->transfer( MCP320X_PACKET_SIZE, this->packet );
	}
	catch( SPIException e ) {
		throw e;
	}
	
	d = (rxbuf[1] & 0x0F) << 8;
	d |= rxbuf[2];
	delete(rxbuf);
	
	return (double) ( (d * ref) / MCP320X_RESOLUTION );
}

MCP320X::~MCP320X(void)
{
	delete(spi);
}
